# HIT docker

### Installation
1. Install [Docker for Mac](https://docs.docker.com/engine/installation/mac/)
2. Create the hit network: `docker network create hit`

### Access Necessary
1. Access to our Private Docker Hub Organization
2. Access to our NPM Organization

### Build & run apps
1. Compile your server and front-end code by doing the following: `npm run compile:ts && npm run build:apps`
2. Run all scripts/build from `docker` directory
3. `./scripts/build.sh` builds all containers (this needs to be run separately from start)
  * Only need to re-run when there is a change to the container build (`Dockerfile`)
  ** for more control please look at scripts/build.sh for options catering the developer env
4. `docker-compose up -d` starts up all containers in the background
   ** It will use image local copies. It will start services in interactive mode (non-demon) and it will mount local dirs (everythign in your source dir including node_modules)
5. Files on the host are mapped into all containers
   * Static files (in Nginx) will not require restarting the container
   * To restart a Node container to pick up changes, run `docker-compose restart <service_name>`
   * To recreate and restart a container (to pick up config changes in `docker-compose.yml`), run `docker-compose up -d <service_name>`
6. To tail logs for a particular container: `docker-compose logs --follow <service_name>`
7. Poke around container with `docker-compose exec <container> <command>` (ex. sh)
8. Stop all containers with `docker-compose stop`

### Regular cleanup
1. Periodically clean up old images/volumes with `./scripts/cleanup.sh`
2. You can also manually clean up images on your host with `docker images` -> `docker rmi <image>`

### Adding new services
1. Development: add service and appropriate linkages in `docker-compose.yml`
2. Cloud: deployments need configuration changes in the `deployments` folder and setup for app creation. Please check with KPK!

### Database
1. Database volume is mounted at `docker/.postgres`. You can blow away the data directory and restart the container to start fresh.
2. `docker-compose exec database psql -U nodejs -d dev` to interact with the Postgres database

## Redis
1. Redis volume is mounted at `docker/.redis`. You can blow away the data directory and restart the container to start fresh.
2. `docker-compose exec redis redis-cli` to interact with Redis cache

### Testing
1. GitLab CI eslint is run in a pre-packaged container. To run the same setup on your machine, run `./scripts/lint.sh`

### Debugging using Webstorm (or other IDE)
1.  Run following script from `docker/` dir:
    ```
    ./scripts/node-inspect.sh -d <appname> <appname>
    ```
    That will start up your application(s) in "detached" mode using node's `--inspect` flag.

2.  Create a new Node.js Remote Debug entry with host: `localhost`, debug port = host port (listed in `docker-compose-node-debug.yml` in range starting at `5858`). Also be sure to select the
    inspect option.

### Debugging using Chrome
1.  Run following script from `docker/` dir:
    ```
    ./scripts/node-inspect.sh -d <appname> <appname>
    ```
    That will start up your application(s) in "detached" mode using node's `--inspect` flag.

2.  Tail your application's logs to find your Chrome inspector link with the following command:
    ```
    docker-compose logs --tail 300 -f <appname>
    ```

3.  You will have the following logs while the application boots up:

    ```
    haven_1                    | Use Inspector Port: 5866
    haven_1                    | Debugger listening on port 5858.
    haven_1                    | Warning: This is an experimental feature and could change at any time.
    haven_1                    | To start debugging, open the following URL in Chrome:
    haven_1                    |     chrome-devtools://devtools/remote/serve_file/@60cd6e859b9f557d2312f5bf532f6aec5f284980/inspector.html?experiments=true&v8only=true&ws=127.0.0.1:5858/00d4c00c-5c85-4a2a-9cba-4bb462d49811
    ```

    You will need to copy and paste that `chrome-devtools://` link into your Chrome browser and update the port listed in `ws=127.0.0.1:5858` to be the port specified by `Use Inspector Port: 5866`. For example, this link would be updated to:

    ```
    chrome-devtools://devtools/remote/serve_file/@60cd6e859b9f557d2312f5bf532f6aec5f284980/inspector.html?experiments=true&v8only=true&ws=127.0.0.1:5866/00d4c00c-5c85-4a2a-9cba-4bb462d49811
    ```

    Once you go to that link, you should see the following verification that the debugger is attached and ready: `Debugger attached.`

### Testing with all dependencies
1.  `./docker/docker-compose-dependencies.yml` has been added for haven/coverpath testing

2.  The additional apps in the dependencies yml file are
    ```
    policy-admin
    policy-admin-api
    policy-admin-scheduler
    policy-admin-robot
    policy-admin-startup
    pdfstream
    cdm-producers
    ```
    The above list are a soft dependency - ie: you need to use these names when you docker compose up (see below)
    ```
    docker-compose -f docker-compose.yml -f docker-compose-dependencies.yml up coverpath-agent coverpath-client admin hit-workflow uw-workflow product product-startup workflow hit-data-svc coverpath-agent coverpath-client pdfstream cdm-producers policy-admin
    ```

3.  The images are pulled using latest development/master builds from our gitlab registry.

4.  Each dependency (in addition to images in the main compose file) can be overridden using env vars during launch
    As an example here is the policy-admin service in the dependencies yml
    ```
    policy-admin:
        image: ${IMAGE_PA:-registry.gitlab.havenlife.com/hit/policy-admin:development}
    ```

    If `IMAGE_PA` is not set it will pull/use `development` pa build. to override this and use say a production version from docker hub you run docker-compose like this
    ```
    IMAGE_PA=havenlife/policy-admin/release-candidate-20190410-clean-the-steam-wand-1.29.0_3 docker-compose -f docker-compose.yml -f docker-compose-dependencies.yml up policy-admin
    ```

    This is use/pull a diff image to launch policy-admin in your tests.
    NOTE: Notice the two yml files pulled in `docker-compose -f docker-compose.yml -f docker-compose-dependencies.yml`

5.  You have to use two (2) compose files. This is the same layering strategy we use when we override ports for debugging etc... There is no
    need to change the main docker-compose file to your liking and affect all devs.
    ```
    docker-compose -f docker-compose.yml -f docker-compose-dependencies.yml up haven workflow etc...
    ```

6.  The main docker-compose.yml file already brings in product as a hard dependency.

7.  Local ports for postgres/redis/rabbit have been removed to take advantage of docker and to avoid silly collisions on dev machines. If
    you want to expose local ports use `docker-compose-dev-local-ports.yml` as part of the command.

#### Additional notes
1. `package.json` is now pegged at specific versions (no semver ranges). Please add any new dependencies in the same manner to ensure that installation caching works properly.
