#!/usr/bin/env bash

tag=development
app=eventor-bridges/eventor-kafka-elastic-bridge
builds=(
    "debian"
)

devbuild=true
cleanup=true

debian_args="--build-arg BUILD_AMORPHIC=yes"

org="local"

while getopts ":b:d:t:o:Dxyz" opt; do
  case $opt in
    b) IFS=', ' read -r -a builds <<< "$OPTARG"
    ;;
    t) tag="${OPTARG}"
    ;;
    d) debian_args+=" --build-arg ${OPTARG}"
    ;;
    o) org=${OPTARG}
    ;;
    D)
        cleanup=false
    ;;
    x)
        debian_args+=" --no-cache"
    ;;
    y)
        debian_args+=" --build-arg BUILD_SOURCE=yes"
    ;;
    z)
        devbuild=false
    ;;
    *) echo "Invalid option" && exit 1
    ;;
  esac
done

if [ "$devbuild" = true ] ; then
        debian_args+=" --build-arg DEV_MOUNT_ONLY_SKIP_NPM_AND_BUILD=yes"
fi

echo "Running build for org [$org] ..."


# additional docker build args
# BUILD_SOURCE will install devdeps and run a comiple step - note that if you compile before docker build you can skip this step.
# The string has to be non-empty
#
# apline_args='--build-arg BUILD_SOURCE=\"F'yeah\" --build-arg BUILD_WEBPACK=\"F'yeah\"
#
# BUILD_WEBPACK will not install devdeps (it's tied to BUILD_SOURCE) and run a webpack build - note that if you build before docker build you can skip this step.
# The string has to be non-empty
# WARNING - this adds 10+ minutes to the initial container build
#
# --build-arg BUILD_WEBPACK=\"F'yeah\"
#
# ***********************
# Nginx container WILL NOT BUILD ANYTHING! - you need to have built before docker build.
#
# if you want to compile and build webpack in the container = slow
#
# ***********************
#
# if you are in dev mode and are always mounting your local node_modules and dist/dist-server dirs then you can skip the npm install in the container
# since these folders will always mount your local and shadow the inner target, use this cmd line ars
#
# --build-arg DEV_MOUNT_ONLY_SKIP_NPM_AND_BUILD=\"fuhgeddaboudit\"
#
# example: debian_args="" will install prod npm BUT will not install dev npm for ts compile or webpack build, it assumes you will mount local dirs that are compiled etc.. (you can mount local node_modules as well)
# example: debian_args="--build-arg BUILD_SOURCE=\"F'yeah\"" will install prod npm AND ts compile BUT will not install dev npm for webpack build, it assumes you will mount dist local dirs that are compiled etc.. (you can mount local node_modules as well)
# example: debian_args="--build-arg DEV_MOUNT_ONLY_SKIP_NPM_AND_BUILD=\"fuhgeddaboudit\"" will skip npm and ts compile, all mounted local = fast build
#

for build in "${builds[@]}"; do
    build_tag="$org"/hit-"$build"/"$app":"$tag"

    echo "Building docker image for $build_tag ..."
    if ! eval eval docker build --force-rm -f "$build"/Dockerfile -t "${build_tag}" \$\{"${build}"_args\} ../ ; then
    	echo >&2 "Build failed."
    	exit 1
    fi
done

echo "Build complete."

if [ "$cleanup" = true ] ; then
    echo "Cleaning up docker litter..."
    # on the heavy images this tries to delete too soon so lets pause
     # exit 0 here - if we got here the build is ok... dont stop the show for cleanup
    sleep 5
    docker rmi $(docker images -a|grep "<none>"|awk '$1=="<none>" {print $3}') 2>/dev/null
    exit 0
    
    #docker rmi "$(docker images | awk '$3 == "<none>" { print $3 }')"
    #echo "Cleanig up after ourselves..."
    #docker rmi $(docker images --filter dangling=true -q --no-trunc)
fi
