#!/usr/bin/env bash

while getopts ":e:" opt; do
    case $opt in
        e)
            env=${OPTARG}
            ;;
        \?)
            echo "Invalid option: -$OPTARG" >&2
            exit 1
            ;;
        :)
            env="development"
            ;;
    esac
done

tag="workflow-regression"
image="debian,debian_workflow"

silent_cmd() {
    "$@" > /dev/null
}

preBuild() {
    echo 'Running perBuild hook.'
    echo 'preBuild:NO-OP'
}

postBuild() {
    echo 'Running postBuild hook.'
    echo 'postBuild:NO-OP'
}

build() {
    ./scripts/build.sh -t "$tag" -b "$image" -d BUILD_DEV_REGRESSION=yes -d BUILD_SOURCE=yes
}

build_local() {
    ./scripts/build.sh -t "$tag" -b "$image" -d DEV_MOUNT_ONLY_SKIP_NPM_AND_BUILD=yup
}


if [ "$env" = "ci" ]; then
    # Hide Logging Ouput for CI Pipelines
    cd docker && silent_cmd build && cd ..
elif [ "$env" = "local" ]; then
    npm run clean
    npm run reinstall
    npm run compile:ts

    cd docker && build_local && cd ..
else
    npm run clean
    cd docker && build && cd ..
fi
