#!/usr/bin/env bash

regression_svc='workflow-regression-hit';
env='development'
mode='normal'
debug_output=false;
prefix=''
inspect=''

while getopts ":e:m:s:p:di" opt; do
    case $opt in
        e)
            env=${OPTARG}
            ;;
        m)
            mode=${OPTARG}
            ;;
        d)
            debug_output=true;
            ;;
        s)
            regression_svc=${OPTARG}
            ;;
        p)
            prefix=${OPTARG}
            ;;
        i)
            inspect=true;
            ;;
        \?)
            echo "Invalid option: -$OPTARG" >&2
            exit 1
            ;;
        :)
            env="development"
            ;;
    esac
done

if [ -z "$prefix" ]; then
    echo "You need to provide a prefix using -p <string>"
    exit 1;
fi

echo "Running regression - service[${regression_svc}], env[${env}], mode[${mode}]";

docker network create "${prefix}-regression" || true
docker-compose -f ./docker/docker-compose-test.yml kill
docker-compose -f ./docker/docker-compose-test.yml rm -s -v -f product database-regression || true
docker-compose -f ./docker/docker-compose-test.yml pull product database-regression || true

if [ "$env" = "ci" ]; then
    if [ "$mode" = "mini" ]; then
        docker-compose -f ./docker/docker-compose-test.yml -f ./docker/docker-compose-test-ci.yml -f ./docker/docker-compose-test-mini.yml up --quiet-pull --force-recreate --remove-orphans --exit-code-from "${regression_svc}" "${regression_svc}"
    else
        docker-compose -f ./docker/docker-compose-test.yml -f ./docker/docker-compose-test-ci.yml up --quiet-pull --force-recreate --remove-orphans --exit-code-from "${regression_svc}" "${regression_svc}"
    fi
elif [ "$env" = "local" ]; then
    if [ "$mode" = "mini" ]; then
        docker-compose -f ./docker/docker-compose-test.yml -f ./docker/docker-compose-test-mount-local.yml -f ./docker/docker-compose-test-mini.yml up --force-recreate --remove-orphans --exit-code-from "${regression_svc}" "${regression_svc}"
    else
        if [ "$inspect" = true ]; then
            docker-compose -f ./docker/docker-compose-test.yml -f ./docker/docker-compose-test-mount-local.yml -f ./docker/docker-compose-test-node-inspect.yml up --force-recreate --remove-orphans --exit-code-from "${regression_svc}" "${regression_svc}"
        else
            docker-compose -f ./docker/docker-compose-test.yml -f ./docker/docker-compose-test-mount-local.yml up --force-recreate --remove-orphans --exit-code-from "${regression_svc}" "${regression_svc}"
        fi
    fi
else
    if [ "$mode" = "mini" ]; then
        docker-compose -f ./docker/docker-compose-test.yml -f ./docker/docker-compose-test-mini.yml up --force-recreate --remove-orphans --exit-code-from "${regression_svc}" "${regression_svc}"
    else
        if [ "$debug_output" = true ]; then
            docker-compose -f ./docker/docker-compose-test.yml up --force-recreate --remove-orphans
        else
            docker-compose -f ./docker/docker-compose-test.yml up --force-recreate --remove-orphans --exit-code-from "${regression_svc}" "${regression_svc}"
        fi
    fi
fi
